/*
 * FileType.java - 
 * Copyright (C) 2000-2001 Iigo Gonzlez
 * sensei@hispavista.com
 * http://www.geocities.com/innigo.geo
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jos.jexplorer;

import java.awt.*;
import javax.swing.*;
import java.util.*;
import java.io.*;
import java.net.*;

/**
 * Contents the Jos file types. This class helps to JExplorer to find the icons for diferents file types.
 * In future version, it will help JExplorer to know the path where find the executable files for this file type.
 * @version 1.0.1
 */
public class FileType{

	private String name = null;
	private ImageIcon bigIcon = null;
	private ImageIcon smallIcon = null;
	private String viewer = null; //viewer name
	private URL url;

	static Hashtable hash = new Hashtable();

	public static FileType DEFAULT_FILE_TYPE = new FileType("Default file", ThemesManager.getImage("bigFile.gif"));
	public static FileType DEFAULT_FOLDER_TYPE = new FileType("Default folder", ThemesManager.getImage("bigFolder.gif"));

	static {
		try {
			File file = new File(System.getProperty("user.dir") + "/resources/fileTypes.properties");
			FileInputStream fis = new FileInputStream(file);
			PropertyResourceBundle prop = new PropertyResourceBundle(fis);
			for(Enumeration e = prop.getKeys();e.hasMoreElements();){
				String key = (String)e.nextElement();
				String value = prop.getString(key);
				int labelComma = value.indexOf(",");
				if (labelComma != -1){
					String label = value.substring(0, labelComma);
					int iconComma = value.indexOf(",", labelComma + 1);
					if (iconComma != -1){
						String path = value.substring(labelComma + 1, iconComma);
						if (iconComma < value.length()){
							String view = value.substring(iconComma + 1);
							hash.put(key, new FileType(label, path, view));
						} else {
							hash.put(key, new FileType(label, path));
						}
					} else {
						String path = value.substring(labelComma + 1);
						hash.put(key, new FileType(label, path));
					}
				} else {
					hash.put(key, new FileType(value));
				}
			}
		} catch (FileNotFoundException fnfe){
			JOptionPane.showMessageDialog(null, I18n.getString("ERROR_FILETYPE_NOT_FOUND"), I18n.getString("ERROR_TITLE"), JOptionPane.ERROR_MESSAGE);
			//fnfe.printStackTrace();
		} catch (IOException ioe){
			ioe.printStackTrace();
		} catch (MissingResourceException mre) {
			mre.printStackTrace();
		}
	};

	/**
	 * Creates a filetype with the specific name.
	 * @param name the type name
	 */
	private FileType(String name){
		this(name, null, null);
	}

	/**
	 * Creates a filetype with the specific name and icon.
	 * @param name the type name
	 * @param path the icon path
	 */
	private FileType(String name, String path){
		this(name, path, null);
	}

	/**
	 * Creates a filetype with the specific name and icon path.
	 * @param name the type name
	 * @param path the icon path
	 * @param viewer viewer name (class name)
	 */
	private FileType(String name, String path, String viewer){
		this.name = name;
		if (path != null){
			//bigIcon = I18n.getImageIcon(this, path);
			bigIcon = new ImageIcon(System.getProperty("user.dir") + File.separator + path);
			smallIcon = new SmallIcon(bigIcon);
		}
		this.viewer = viewer;
	}

	/**
	 * Creates a filetype with the specific name and icon image.
	 * @param name the type name
	 * @param image the icon image
	 */
	private FileType(String name, ImageIcon image){
		this.name = name;
		if (image != null){
			bigIcon = image;
			smallIcon = new SmallIcon(bigIcon);
		}
		this.viewer = null;
	}

	/**
	 * finds the file type for a given type. This type, usually, will be a file extension
	 * @param type file type or file extension
	 * @return the file type of a given extension
	 */
	public static FileType getFileType(String type){
		return (FileType)hash.get(type);
	}
	
	public String getTypeName(){
		return name;
	}

	public String getViewerName(){
		return viewer;
	}

	public Icon getBigIcon(){
		return bigIcon;
	}

	public Icon getSmallIcon(){
		return smallIcon;
	}
	
	/**
	 * Paints a small icon from the default icon.
	 */
	class SmallIcon extends ImageIcon{
		public static final int SMALL_WIDTH = 16;
		public static final int SMALL_HEIGHT = 16;

		private ImageIcon icon;

		public SmallIcon(ImageIcon icon){
			this.icon = icon;
		}

		public int getIconWidth(){
			return SMALL_WIDTH;
		}

		public int getIconHeight(){
			return SMALL_HEIGHT;
		}

		public void paintIcon(Component c, Graphics g, int x, int y){
			g.drawImage(icon.getImage(), x, y, SMALL_WIDTH, SMALL_HEIGHT, null);
		}
	}
}
